"""gitea api操作
主要功能：
1.判断指定的branch/tag/commit是否存在；
2.创建tag;
3.创建release;
"""
# coding = utf-8
import os
import re
import json
import logging
import requests
from .git import parser_git_url
from .my_http import format_output


__all__ = ["is_checkout", "tetrieve_branch", "get_tags", "get_commit_default"]

logger = logging.getLogger("DengUtils")

DOMAIN = os.environ["GIT_CI_API_URL"]
ACCESS_TOKEN = os.environ["GIT_CI_TOKEN"]

# gitea api存在bug，apidoc中标明支持headers方式鉴权，但实际不支持
# auth_header = {"Authorization": ACCESS_TOKEN, "accept": "application/json; charset=UTF-8"}
auth_params = {"access_token": ACCESS_TOKEN}

session = requests.Session()
# session.headers.update(auth_header)


def checkout_response(response: requests.Response, level=""):
    """检查请求响应"""
    if response.status_code == 403 or response.status_code == 401:
        format_output(response, "error", depth=3)
        raise ValueError("权限认证失败，请检查！")

    if response.status_code >= 400:
        format_output(response, "error", depth=3)
    else:
        if level:
            format_output(response, level=level, depth=3)
        return response.json()


def is_checkout(*, owner: str, repo: str, target: str):
    """提供的branch/tag/commitId是否存在"""
    # gitea中允许branch与tag重名，必须区分优先级
    # tag优先级最高
    _res_tag = get_tags(owner=owner, repo=repo)
    if _res_tag:
        for _tag in _res_tag:
            if _tag["name"] == target:
                return _tag

    # branch优先级第二
    _res_branch = tetrieve_branch(owner=owner, repo=repo, branch=target)
    if _res_branch:
        return _res_branch

    # commit 优先级最低
    _res_commit = get_commit_default(owner=owner, repo=repo, sha=target)
    if _res_commit:
        return _res_commit

    logger.warning(f"在{owner}/{repo}的tag/branch/commit中找不到目标：{target}")
    return None


def is_checkout_by_obj(repos: dict, target: str):
    """提供的branch/tag/commitId是否存在"""
    # target预处理，target中可能带有前缀
    if ":" in target:
        _type, target = target.split(":", 1)
    else:
        _type = None

    # gitea中允许branch与tag重名，必须区分优先级
    # tag优先级最高
    owner, repo = parser_git_url(repos["repo_url"])
    _res_tag = get_tags(owner=owner, repo=repo)
    if _res_tag:
        for _tag in _res_tag:
            if _tag["name"] == target:
                repos["branch"] = f"TAG:{target}"
                return True, owner, repo, target, repos

    # branch优先级第二
    _res_branch = tetrieve_branch(owner=owner, repo=repo, branch=target)
    if _res_branch:
        repos["branch"] = f"BRANCH:{target}"
        return True, owner, repo, target, repos

    # commit 优先级最低
    _res_commit = get_commit_default(owner=owner, repo=repo, sha=target)
    if _res_commit:
        repos["branch"] = f"COMMIT:{target}"
        return True, owner, repo, target, repos

    logger.error(f"在{owner}/{repo}的tag/branch/commit中找不到目标：{target}")
    repos["branch"] = None
    return False, owner, repo, target, repos


def get_protection_branch(*, owner: str, repo: str, branch: str):
    """获取受保护分支"""
    _url = f"{DOMAIN}/repos/{owner}/{repo}/branch_protections/{branch}"
    _res = session.get(_url, params=auth_params)
    return checkout_response(_res)


def protection_branch(*, owner: str, repo: str, branch: str):
    """保护分支"""
    _url = f"{DOMAIN}/repos/{owner}/{repo}/branch_protections"
    payload = {
        "branch_name": branch,
        "enable_push": False,
        "enable_push_whitelist": False,
        "push_whitelist_usernames": [],
        "push_whitelist_teams": [],
        "push_whitelist_deploy_keys": False,
        "enable_merge_whitelist": True,
        "merge_whitelist_usernames": [],
        "merge_whitelist_teams": ["codereview-group"],
        "enable_status_check": False,
        "status_check_contexts": None,
        "required_approvals": 1,
        "enable_approvals_whitelist": True,
        "approvals_whitelist_username": [],
        "approvals_whitelist_teams": ["codereview-group"],
        "block_on_rejected_reviews": False,
        "block_on_outdated_branch": False,
        "dismiss_stale_approvals": False,
        "require_signed_commits": False,
        "protected_file_patterns": "",
    }
    _res = session.post(_url, params=auth_params, json=payload)
    return checkout_response(_res, level="info")


def cancel_protection_branch(*, owner: str, repo: str, branch: str):
    """取消分支保护"""
    _url = f"{DOMAIN}/repos/{owner}/{repo}/branch_protections/{branch}"
    _res = session.delete(_url, params=auth_params)
    if _res.status_code == 204:
        return True


def delete_branch(*, owner: str, repo: str, branch: str):
    """删除分支"""
    _url = f"{DOMAIN}/repos/{owner}/{repo}/branches/{branch}"
    assert branch not in ("master", "release")
    _res = session.delete(_url, params=auth_params)
    if _res.status_code == 204:
        return True


def tetrieve_branch(*, owner: str, repo: str, branch: str):
    """检索特定分支"""
    _url = f"{DOMAIN}/repos/{owner}/{repo}/branches/{branch}"
    _res = session.get(_url, params=auth_params)
    return checkout_response(_res, level="debug")


def create_branch(*, owner: str, repo: str, old_branch_name: str, new_branch_name: str):
    """创建新分支"""
    # 检查新分支是否存在，存在是直接返回
    new_branch = tetrieve_branch(owner=owner, repo=repo, branch=new_branch_name)
    if new_branch:
        return True, new_branch

    # 不存在时创建分支
    _url = f"{DOMAIN}/repos/{owner}/{repo}/branches"
    assert new_branch_name not in ("master", "release")
    payload = {"old_branch_name": old_branch_name, "new_branch_name": new_branch_name}
    _res = session.post(_url, params=auth_params, json=payload)
    if _res.status_code == 404:
        raise ValueError(f"{owner}/{repo}库找不到源分支：{old_branch_name}")
    data = _res.json()
    data["owner"] = owner
    data["repo"] = repo
    if _res.status_code == 201:
        return True, data
    else:
        return False, data


def get_tags(*, owner: str, repo: str):
    """获取库中tag标签列表"""
    _url = f"{DOMAIN}/repos/{owner}/{repo}/tags"
    _res = session.get(_url, params=auth_params)
    return checkout_response(_res)


def get_commit_default(*, owner: str, repo: str, sha: str):
    """获取commit详情"""
    _url = f"{DOMAIN}/repos/{owner}/{repo}/git/commits/{sha}"
    _res = session.get(_url, params=auth_params)
    return checkout_response(_res, level="debug")


def release_exists(
    *, owner: str, repo: str, name: str, tag_name: str, target: str
) -> dict:
    """判断release是否已经存在"""
    _release_list = get_releases(owner=owner, repo=repo)
    for _release in _release_list:
        if (
            _release["name"] == name
            and _release["tag_name"] == tag_name
            and _release["target_commitish"] == target
        ):
            logger.info(f"release已经存在：{owner}/{repo}:{name}")
            return _release
    logger.info(f"release不存在：{owner}/{repo}:{name}")
    return {}


def get_releases(*, owner: str, repo: str, page=1, per_page=100):
    """获取release列表"""
    _url = f"{DOMAIN}/repos/{owner}/{repo}/releases"
    params = {**auth_params, "page": page, "per_page": per_page}
    _res = session.get(_url, params=params)
    return checkout_response(_res)


def get_release_by_id(*, owner: str, repo: str, release_id: int):
    """获取release详情"""
    _url = f"{DOMAIN}/repos/{owner}/{repo}/releases/{release_id}"
    _res = session.get(_url, params=auth_params)
    return checkout_response(_res, level="debug")


def create_release(*, owner: str, repo: str, payload: dict) -> dict:
    """创建release"""
    # 先判断相应的release是否已经存在
    _release = release_exists(
        owner=owner,
        repo=repo,
        name=payload["name"],
        tag_name=payload["tag_name"],
        target=payload["target_commitish"],
    )
    if _release:
        return _release

    # 不存在时创建
    _url = f"{DOMAIN}/repos/{owner}/{repo}/releases"
    logger.info(
        f"【{owner}/{repo}】库发布版本，参数：{json.dumps(payload, ensure_ascii=False, indent=4)}"
    )
    _res = session.post(_url, params=auth_params, json=payload)
    return checkout_response(_res, level="info")


def batch_delete_release(release_list: list):
    """批量删除release"""
    for _release in release_list:
        _res = session.delete(_release["url"], params=auth_params)
        if _res.status_code == 204:
            logger.info(f"删除成功：{_release['url']}")
        elif _res.status_code == 404:
            logger.warning(f"记录不存在：{_release['url']}")
        else:
            format_output(_res)
            logger.error(f"删除失败：{_release['url']}")


def delete_release(*, owner: str, repo: str, release_id: int):
    """删除release"""
    release_obj = get_release_by_id(owner=owner, repo=repo, release_id=release_id)
    assert release_obj["id"] == release_id
    assert release_obj["prerelease"] is True, "非预发布状态，不能删除"
    assert release_obj["name"].startswith("temp_"), "release名字校验不通过，只有以temp_开头的才能删除"
    logger.info(f"可删除性校验通过，开始删除release: {release_obj['name']}")
    _url = f"{DOMAIN}/repos/{owner}/{repo}/releases/{release_id}"
    _res = session.delete(_url, params=auth_params)
    return checkout_response(_res, level="info")


def get_pulls(
    *, owner: str, repo: str, state: str, sort: str = "recentupdate", page=1, limit=50
):
    """获取pull列表"""
    _url = f"{DOMAIN}/repos/{owner}/{repo}/pulls"
    if isinstance(page, int):
        params = {"state": state, "sort": sort, "page": page, "limit": limit}
    else:
        params = {"state": state, "sort": sort, "page": 1, "limit": limit}
    params.update(auth_params)
    pull_list = []
    while True:
        _res = session.get(_url, params=params)
        _temp_pulls = checkout_response(_res)
        pull_list.extend(_temp_pulls)
        if len(_temp_pulls) < limit or isinstance(page, int):
            break
        else:
            params["page"] += 1
    return pull_list


def merge_audit(
    *,
    owner: str,
    repo: str,
    index: int,
    audit: str = "merge",
    title: str = "自动合并",
    body: str = "来自web-hock的自动合并",
):
    """审核Merge Request"""
    _url = f"{DOMAIN}/repos/{owner}/{repo}/pulls/{index}/merge"
    payload = {
        "Do": audit,
        "MergeMessageField": body,
        "MergeTitleField": title,
        "force_merge": False,
    }
    _res = session.post(_url, params=auth_params, json=payload)
    return _res.status_code


def get_pull_by_issue_id(*, owner: str, repo: str, issue_id: int):
    """获取指定issue详情"""
    _url = f"{DOMAIN}/repos/{owner}/{repo}/pulls/{issue_id}"
    _res = session.get(_url, params=auth_params)
    return checkout_response(_res, level="debug")


def merge_new(
    *,
    owner: str,
    repo: str,
    target: str,
    source: str,
    title: str = "hock自动发起",
    body: str = "hock监控到有往release分支的merge request时，自动往master上合并",
    exist_ok: bool = True,
):
    """发起merge request"""
    payload = {
        "base": target,
        "head": source,
        "body": body,
        "title": title,
    }
    _url = f"{DOMAIN}/repos/{owner}/{repo}/pulls"
    _res = session.post(_url, params=auth_params, json=payload)
    if (
        exist_ok
        and _res.status_code == 409
        and "pull request already exists for these targets" in _res.text
    ):
        logger.info("相应Merge Request 已经存在，无需再次创建，直接获取已经存在的内容")
        format_output(_res, "debug")
        _e = r"issue_id: (\d+?),"
        _r = re.search(_e, _res.text, re.I)
        issue_id = int(_r.groups()[0])
        return get_pull_by_issue_id(owner=owner, repo=repo, issue_id=issue_id)
    return checkout_response(_res, level="debug")


if __name__ == "__main__":
    pass



import os
import sys
import json
import time
import ctypes
import logging
from pathlib import Path
from urllib.parse import urlparse
from .utils import execute_cmd, byte_to_str, bytes_to_str, ExecuteCMDException


logger = logging.getLogger("DengUtils")


def get_system_info(win_rm_session=None) -> dict:
    """获取系统信息"""
    # 本地缓存路径
    __system_path = Path(os.environ.get("TEMP") or os.environ.get("HOME")) / "system_info.json"
    # 本地时优化检查缓存，远程不存在缓存一说
    if not win_rm_session:
        _system_info: dict
        if __system_path.exists():
            with open(__system_path, encoding="utf-8") as _file:
                try:
                    _system_info = json.load(_file)
                except Exception as e:
                    logger.exception(e)
                    logger.warning(f"读取缓存出错，重新计算：{str(e)}")
                else:
                    for _key, _value in _system_info.items():
                        if not _value:
                            logger.warning(f"缓存信息不完整，重新计算：{_system_info}")
                            break
                    else:
                        timestamp = _system_info.pop("timestamp")
                        if int(time.time()) - timestamp <= 1 * 24 * 60 * 60:
                            logger.debug(
                                f"系统信息缓存未过期：{json.dumps(_system_info, ensure_ascii=False, indent=4)}"
                            )
                            return _system_info
                        else:
                            logger.info(f"缓存过期，重新获取")

    # 实时获取系统信息
    systems = {
        "host_name": "",
        "os_name": "",
        "os_version": "",
        "os_arch": "",
        "ip_address": "",
        "mac_address": "",
    }
    try:
        if win_rm_session:
            res = win_rm_session.run_cmd("systemInfo")
            system_info = bytes_to_str(res.std_out)
        else:
            res = execute_cmd(["systemInfo"], encoding=None, level=None)
            system_info = bytes_to_str(res.stdout)
        for line in system_info.split("\n"):
            line = line.strip()
            if (
                line.startswith("主机名:")
                or line.startswith("主機名稱:")
                or line.startswith("Host Name:")
            ):
                systems["host_name"] = line.split(":")[-1].strip()
            elif (
                line.startswith("OS 名称:")
                or line.startswith("作業系統名稱:")
                or line.startswith("OS Name:")
            ):
                systems["os_name"] = line.split(":")[-1].strip()
            elif (
                line.startswith("OS 版本:")
                or line.startswith("作業系統版本:")
                or line.startswith("OS Version:")
            ):
                systems["os_version"] = line.split(":")[-1].strip()
            elif (
                line.startswith("系统类型:")
                or line.startswith("系統類型:")
                or line.startswith("System Type:")
            ):
                systems["os_arch"] = line.split(":")[-1].strip()
        try:
            from . import net as net_utils
        except ImportError:
            import net as net_utils
        if win_rm_session:
            systems["ip_address"] = urlparse(win_rm_session.url).netloc.split(":")[0]
        else:
            systems["ip_address"] = net_utils.get_host_ip()
        systems["mac_address"] = get_mac_address(systems["ip_address"], win_rm_session)
        logger.info(f"当前机器系统信息：{json.dumps(systems, ensure_ascii=False, indent=4)}")
        if not win_rm_session:
            with open(__system_path, mode="w", encoding="utf-8") as _file:
                _temp = systems
                _temp["timestamp"] = int(time.time())
                json.dump(_temp, _file, ensure_ascii=False, indent=4)
        return systems
    except Exception as e:
        logger.exception(e)
        return systems


def get_mac_address(ip_address: str, win_rm_session=None) -> str:
    """根据IP地址获取网卡MAC地址，兼容中文简体、中文繁体、英文
    :param ip_address: str, 目标网卡对应的IP地址
    :param win_rm_session: Windows 远程管理服务会话对象
    """
    mac_address: str = ""
    try:
        if win_rm_session:
            res = win_rm_session.run_cmd("ipconfig /all")
            network_info = bytes_to_str(res.std_out)
        else:
            res = execute_cmd(["ipconfig", "/all"], encoding=None, level=None)
            network_info = bytes_to_str(res.stdout)
        for _line in network_info.split("\n"):
            _line = _line.strip()
            if _line:
                if (
                    _line.startswith("Physical Address")
                    or _line.startswith("物理地址")
                    or _line.startswith("實體位址")
                ):
                    mac_address = _line.split(":")[-1].strip()
                elif (
                    _line.startswith("IPv4 Address")
                    or _line.startswith("IPv4 地址")
                    or _line.startswith("IPv4 位址")
                ):
                    if ip_address in _line:
                        return mac_address
            else:
                # 切换网卡、重置mac
                mac_address = ""
    except Exception as e:
        logger.exception(e)

    return ""


def is_running_as_admin() -> bool:
    """检查当前是否以管理员权限运行"""
    try:
        return bool(ctypes.windll.shell32.IsUserAnAdmin())
    except:
        return False


def get_current_app_list(detail=False, win_rm_session=None, timeout: int = 60) -> list:
    """获取当前用户实时app列表"""
    cmd_text = 'tasklist /nh /fo csv /fi "SESSION ne 0"'
    if detail:
        cmd_text += " /v"

    if win_rm_session:
        _res = win_rm_session.run_cmd(cmd_text, timeout=timeout)
        out_decode = bytes_to_str(_res.std_out)
    else:
        _res = execute_cmd(cmd_text, encoding=None, level=None, timeout=timeout)
        out_decode = bytes_to_str(_res.stdout)

    _app_list = []
    for _line in out_decode.strip().split("\n"):
        _app_info = _line.strip().strip('"').split('","')
        if _app_info:
            try:
                _app = {
                    "name": _app_info[0],
                    "pid": _app_info[1],
                    "session_name": _app_info[2],
                    "session_id": _app_info[3],
                    "mem": _app_info[4],
                }
                if detail:
                    _app["status"] = _app_info[5]
                    _app["username"] = _app_info[6]
                    _app["cpu_time"] = _app_info[7]
                    _app["window_title"] = _app_info[8]
            except Exception as e:
                logger.exception(e)
                logger.error(_line)
            else:
                _app_list.append(_app)
    return _app_list


def kill_pid(pid: int, force: bool = False, timeout: int = 60):
    """kill指定进程"""
    cmd_text = f"taskkill /pid {pid} /t"
    if force:
        cmd_text += " /F"
    try:
        _res = execute_cmd(cmd_text, encoding="GBK", timeout=timeout)
    except ExecuteCMDException as _e:
        if "没有找到进程" in str(_e):
            pass
        else:
            logger.error(_e)


def kill_process(name: str, force: bool = False, timeout: int = 60):
    """kill指定进程"""
    cmd_text = f"taskkill /im {name} /t"
    if force:
        cmd_text += " /F"
    try:
        _res = execute_cmd(cmd_text, encoding="GBK", timeout=timeout)
    except ExecuteCMDException as _e:
        if "没有找到进程" in str(_e):
            pass
        else:
            logger.error(_e)


def get_exe_name_by_pid(pid: int) -> str:
    """通过进程PID获取EXE名称"""
    _cmd = f'tasklist /nh /V /fi "PID eq {pid}"'
    try:
        _res = os.popen(_cmd)
        _info = _res.read()
        _list = _info.split()
        if int(_list[1]) == int(pid):
            return _list[0]
    except Exception as _e:
        logger.exception(_e)
        logger.error(f"执行cmd命令出错：{_cmd}")


def is_32_python_on_window() -> bool:
    """判断当前是否为32位的Python"""
    return "32 bit" in sys.version



import time
import math
import pytz
import datetime
from typing import Union


def get_timestamp(length: int = 10, utc=False) -> int:
    """获取系统时间时间戳"""
    assert length in (10, 13), "时间戳长度参数错误，只有10位与13位时间戳"
    if utc:
        current_timestamp = datetime.datetime.utcnow().timestamp()
    else:
        current_timestamp = datetime.datetime.now().timestamp()
    return int(str(current_timestamp * 1000)[:length])


def get_current_time(
    _format: str = "long", offset: int = 0, sep1="-", sep2=":", **kwargs
) -> str:
    if not isinstance(offset, int):
        raise ValueError(f"offset参数非法，预期为int类型，实际为{type(offset)}")

    now_time = datetime.datetime.now()
    target_time = now_time - datetime.timedelta(seconds=offset)
    if _format.lower() == "short":
        return target_time.strftime(f"%Y{sep1}%m{sep1}%d")
    elif _format.lower() == "long":
        return target_time.strftime(f"%Y{sep1}%m{sep1}%d %H{sep2}%M{sep2}%S")
    elif _format.lower() == "max":
        return target_time.strftime(f"%Y{sep1}%m{sep1}%d %H{sep2}%M{sep2}%S.%f")
    else:
        return target_time.strftime(_format)


def str_to_time(
    time_str: str, _format="long"
) -> Union[datetime.datetime, datetime.date]:
    if not time_str:
        raise ValueError(f"time_str参数非法：{time_str}")

    if _format == "long":
        return datetime.datetime.strptime(time_str, "%Y-%m-%d %H:%M:%S")
    else:
        day = datetime.datetime.strptime(time_str, "%Y-%m-%d")
        return day.date()


def time_to_str(_time: datetime.datetime, _format="long") -> str:
    if _format == "long":
        return _time.strftime("%Y-%m-%d %H:%M:%S")
    else:
        return _time.strftime("%Y-%m-%d")


def timestamp_to_str(
    timestamp: int, fmt="%Y-%m-%d %H:%M:%S", tz: str = "Asia/Shanghai"
) -> str:
    """时间戳转换时间字符串"""
    if len(str(timestamp)) == 13:
        time_int = math.floor(int(timestamp) / 1000)
    elif len(str(timestamp)) == 10:
        time_int = int(timestamp)
    else:
        raise ValueError(f"时间戳格式错误：{timestamp}")

    utc_time = datetime.datetime.fromtimestamp(time_int, tz=pytz.UTC)
    target_time = utc_time.astimezone(pytz.timezone(tz))
    return target_time.strftime(fmt)


def str_to_timestamp(time_str: str, fmt: str) -> float:
    """时间字符串转换时间戳"""
    time_obj = datetime.datetime.strptime(time_str, fmt)
    return time_obj.timestamp()



import sys
import logging
import logging.handlers
from typing import List, Union
from pathlib import Path


__version__ = "2025.10.11.1"
log_format = logging.Formatter(
    "[%(asctime)s] %(threadName)s/%(name)s/%(filename)s/%(lineno)s/%(funcName)s/%(levelname)s: %(message)s"
)

# 日志句柄
logger = logging.getLogger("DengUtils")
logger.propagate = False
# 此处必须为DEBUG，否则在UiBot中多次加载时会覆盖configure_logger中level配置
logger.setLevel(logging.DEBUG)

# 防止多次加载时产生重复的StreamHandler handler
if len(logger.handlers) == 0:
    console_handler = logging.StreamHandler(stream=sys.stdout)
    console_handler.setFormatter(log_format)
    console_handler.setLevel(logging.INFO)
    logger.addHandler(console_handler)


def clean_repeat_handler():
    """清理重复的日志handler"""
    from deng.utils import get_caller_info

    _info = get_caller_info(2)
    logger.debug(f"调用来源：{_info['output']}")

    logger.debug(f"开始清理重复的日志句柄：{logger.handlers}")

    _first_console = False
    # 收集已有的 FileHandler 处理器输出文件路径列表
    log_file_set = set()
    for _handler in logger.handlers:
        if isinstance(_handler, logging.FileHandler):
            if _handler.baseFilename in log_file_set:
                _handler.flush()
                _handler.close()
                logger.removeHandler(_handler)
                logger.debug(f"关闭重复的：{_handler}")
            else:
                log_file_set.add(_handler.baseFilename)
        elif isinstance(_handler, logging.StreamHandler):
            # 对于StreamHandler实例，仅保留一个，防止越积越多
            if _first_console is False:
                _first_console = True
                logger.debug(f"保留：{_handler}")
            else:
                _handler.flush()
                _handler.close()
                logger.removeHandler(_handler)
                logger.debug(f"关闭：{_handler}")
        else:
            logger.debug(f"忽略：{_handler}")


def close_all_handler():
    """关闭所有日志handler"""
    from deng.utils import get_caller_info

    _info = get_caller_info(2)
    logger.debug(f"调用来源：{_info['output']}")

    for handler in logger.handlers:
        handler.flush()
        handler.close()
        logger.removeHandler(handler)


def clean_handler(close_console_handler: bool = False):
    """清理日志handler，关闭所有FileHandler实例，StreamHandler实例只保留一个"""
    from deng.utils import get_caller_info

    _info = get_caller_info(2)
    logger.debug(f"调用来源：{_info['output']}")

    _first_console = False
    logger.debug(f"开始清理日志句柄：{logger.handlers}")
    for _handler in logger.handlers:
        if isinstance(_handler, logging.FileHandler):
            logger.debug(f"关闭：{_handler}")
            _handler.flush()
            _handler.close()
            logger.removeHandler(_handler)
        elif isinstance(_handler, logging.StreamHandler):
            # 如果有指定新的console_handler，则关闭老的
            if close_console_handler:
                logger.debug(f"关闭：{_handler}")
                _handler.flush()
                _handler.close()
                logger.removeHandler(_handler)
            else:
                # 对于StreamHandler实例，仅保留一个，防止越积越多
                if _first_console is False:
                    _first_console = True
                    logger.debug(f"保留：{_handler}")
                else:
                    logger.debug(f"关闭：{_handler}")
                    _handler.flush()
                    _handler.close()
                    logger.removeHandler(_handler)
        else:
            logger.debug(f"忽略：{_handler}")


def configure_logger(
    level: int = None,
    handlers: List[logging.Handler] = None,
    log_file: Union[Path, str] = None,
    append: bool = True,
    caller_info: dict = None,
    unify_level: bool = False,
    fmt: logging.Formatter = None,
):
    """配置日志处理器
    :param level: int, 指定日志级别
    :param handlers: List[Handler], 日志处理器列表
    :param log_file: 日志文件存储路径
    :param append: 是否为追加模式。当为覆盖模式时会清空已有的handler再添加新的
    :param caller_info: 调用源信息
    :param unify_level: 统一日志级别，为True时各Handler与logger都采用统一的level
    :param fmt: 重置日志输出格式
    """
    from deng.utils import get_caller_info

    _info = caller_info or get_caller_info(2)
    logger.debug(f"调用来源：{_info['output']}")

    # 设置主句柄日志级别
    if level is None:
        pass
    elif level in (
        logging.NOTSET,
        logging.DEBUG,
        logging.INFO,
        logging.WARN,
        logging.WARNING,
        logging.ERROR,
        logging.FATAL,
    ):
        logger.setLevel(level)
    else:
        raise ValueError(f"level参数非法：{level}")

    # 检查是否指定了新的console_handler处理器
    new_console_handler = False
    if handlers is None:
        handlers = list()
    for handler in handlers:
        if type(handler) == logging.StreamHandler:
            new_console_handler = True
            break

    # 重置日志处理器输出格式
    if fmt and isinstance(fmt, logging.Formatter):
        for handler in logger.handlers:
            handler.setFormatter(fmt)

    # 各Handler与Logger采用统一的level级别
    if isinstance(level, int) and unify_level:
        for handler in logger.handlers:
            handler.setLevel(level)

    # 非追加模式时，先清空所有已有的handler
    # 必须放置在设置已有Handler日志级别之后，因为已有的Handler可能被别处引用
    # 如果放置在最前面会导致在别处引用的handler日志级别无法受level参数控制
    if not append:
        clean_handler(new_console_handler)

    # 收集已有的 FileHandler 处理器输出文件路径列表
    log_file_list = []
    for handler in logger.handlers:
        if isinstance(handler, logging.FileHandler):
            log_file_list.append(str(handler.baseFilename))

    # 往日志句柄中添加处理器
    if handlers:
        for handler in handlers:
            # 判断是否重复
            if handler in logger.handlers:
                logger.debug(f"日志处理器已经存在，忽略：{handler}")
                continue

            # StreamHandler 类型的处理器只保留一个
            if type(handler) == logging.StreamHandler:
                if new_console_handler:
                    logger.addHandler(handler)
                    # 各Handler与Logger采用统一的level级别
                    if isinstance(level, int) and unify_level:
                        handler.setLevel(level)
                else:
                    logger.debug(
                        f"StreamHandler 类型处理器已经存在（控制台只保留一个），忽略：{handler}"
                    )
                    continue

            # FileHandler 类型处理器不重复，输出到同一目的地的只保留一个
            if isinstance(handler, logging.FileHandler):
                if str(handler.baseFilename) in log_file_list:
                    logger.debug(f"已经有一个相同目的地的处理器，忽略：{handler}")
                    continue
                else:
                    # 各Handler与Logger采用统一的level级别
                    if isinstance(level, int) and unify_level:
                        handler.setLevel(level)
                    logger.addHandler(handler)
                    log_file_list.append(str(handler.baseFilename))
            elif isinstance(handler, logging.Handler):
                # logger.warning(f"忽略日志Handler：{handler}")
                # 各Handler与Logger采用统一的level级别
                if isinstance(level, int) and unify_level:
                    handler.setLevel(level)
                logger.addHandler(handler)
            else:
                raise TypeError(
                    f"类型错误，预期为{logging.Handler}类型，实际为{type(handler)}类型"
                )

    if log_file:
        if str(log_file) in log_file_list:
            logger.debug(f"已经有一个相同目的地的处理器，忽略：{log_file}")
        else:
            log_file = Path(log_file)
            if not log_file.parent.exists():
                log_file.parent.mkdir(parents=True)

            file_handler = logging.handlers.TimedRotatingFileHandler(
                log_file, when="D", backupCount=15, encoding="utf-8"
            )
            file_handler.setFormatter(log_format)
            file_handler.setLevel(level)
            logger.addHandler(file_handler)
            log_file_list.append(str(log_file))

    logger.debug(f"日志配置完成，当前日志处理器：{logger.handlers}")


logger.info(f"当前deng库版本号：{__version__}")



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""数据库操作
1. db_data_stat: 数据库数据量统计
2. compare_database: 比较两个数据库的表结构，找到差异，以db1为基准，找出db2中缺失的表，以及表结构的差异
@author: deng
@file: db_oper.py
@time: 2024/10/22 下午7:38
"""
import logging
from copy import deepcopy
from typing import Dict, List, Tuple, Union

import pymysql
from pymysql.connections import Connection
from pymysql.cursors import DictCursor
from deng import logger, configure_logger
from deng import my_http as http_utils


configure_logger(level=logging.INFO)


def get_connection(
    host: str,
    port: int,
    username: str,
    password: str,
    database: str = None,
    charset="utf8mb4",
) -> Connection:
    """获取数据库连接"""
    connection = pymysql.connect(
        host=host,
        port=port,
        user=username,
        password=password,
        database=database,
        charset=charset,
        cursorclass=DictCursor,
    )

    return connection


def db_data_stat(
    host: str,
    port: int,
    username: str,
    password: str,
    database: str = None,
    ignore_databases: list = None,
) -> Dict[str, Dict[str, int]]:
    """Database 数据量统计"""
    db_info = dict()
    if ignore_databases is None:
        ignore_databases = [
            "mysql",
            "information_schema",
            "performance_schema",
            "sys",
            "test",
        ]
    connection = get_connection(host, port, username, password, database)

    try:
        with connection.cursor() as cursor:
            # 获取数据库中的Database
            cursor.execute("SHOW DATABASES")
            databases = cursor.fetchall()
            http_utils.format_output(databases, level="debug")

            # 遍历每张表并获取行数
            for _database in databases:
                # 指定了特定数据库时，忽略其余的数据库
                if database and database != _database["Database"]:
                    logger.warning(f"忽略Database: {_database['Database']}")
                    continue

                # 忽略系统数据库
                if _database["Database"] in ignore_databases:
                    logger.warning(f"忽略Database: {_database['Database']}")
                    continue

                # 指定数据库
                cursor.execute(f"USE `{_database['Database']}`")

                # 读取数据库中的表数据
                logger.info(f"开始遍历Database: {_database['Database']}")
                table_info = read_db_by_name(cursor, _database["Database"])
                db_info[_database["Database"]] = table_info
    finally:
        # 关闭数据库连接
        connection.close()

    # 按表名称重新排序
    db_info = dict(sorted(db_info.items()))
    return db_info


def read_db_by_name(cursor: DictCursor, database: str) -> dict:
    """读取数据库中的表数据"""
    table_info = dict()
    # 获取数据库中的所有表
    cursor.execute("SHOW TABLES")
    tables = cursor.fetchall()
    http_utils.format_output(tables, level="debug")

    # 遍历每张表并获取行数
    for table in tables:
        table_name = table[f"Tables_in_{database}"]
        cursor.execute(f"SELECT COUNT(*) as count FROM `{table_name}`")
        result = cursor.fetchone()
        row_count = result["count"]
        table_info[table_name] = row_count
        if row_count > 0:
            logger.info(f"Table: {table_name}, Rows: {row_count}")
        else:
            logger.warning(f"表中没有数据：Table: {table_name}, Rows: {row_count}")
    # 按表名称重新排序
    table_info = dict(sorted(table_info.items()))
    return table_info


def echo_result(db_info: dict):
    """输出结果"""
    # 深度拷贝
    db_info_new = deepcopy(db_info)
    http_utils.format_output(db_info_new, level="info")

    # 提取有数据的表
    for db_name, table_info in db_info_new.items():
        for table_name, row_count in list(table_info.items()):
            if row_count > 0:
                del table_info[table_name]

    http_utils.format_output(db_info_new, level="info")


def get_databases(cursor: DictCursor) -> set:
    cursor.execute("SHOW DATABASES")
    return set(row["Database"] for row in cursor.fetchall())


def get_tables(cursor: DictCursor, database: str) -> set:
    cursor.execute("SHOW TABLES")
    return set(row[f"Tables_in_{database}"] for row in cursor.fetchall())


def compare_columns(table: str, columns1: Union[List, Tuple], columns2: Union[List, Tuple]):
    """比较两个表的字段，以columns1为基准，比较columns2中缺失的字段，以及字段的差异"""
    column_names1 = [column["Field"] for column in columns1]
    column_names2 = [column["Field"] for column in columns2]

    missing_columns = set(column_names1) - set(column_names2)
    if missing_columns:
        logger.warning(f"Table: {table} 中缺失的字段：{missing_columns}")
    else:
        logger.info(f"Table: {table} 中没有缺失字段")

    for column in columns1:
        column_name = column["Field"]
        column2 = next(
            (c for c in columns2 if c["Field"] == column_name), None
        )
        if column2 is None:
            continue

        if column != column2:
            # 字段定义不一致时，找出不一致的地方
            for key, value in column.items():
                if column2[key] != value:
                    logger.warning(f"表【{table}】中字段【{column_name}】的属性【{key}】不一致，标准库为【{value}】，比较库为【{column2[key]}】")
        else:
            logger.info(f"Table: {table} 字段: {column_name} 的定义一致")


def compare_indexes(table: str, indexes1: Union[List, Tuple], indexes2: Union[List, Tuple]):
    """比较两个表的索引，以indexes1为基准，比较indexes2中缺失的索引，以及索引的差异"""
    index_names1 = [index["Key_name"] for index in indexes1]
    index_names2 = [index["Key_name"] for index in indexes2]

    missing_indexes = set(index_names1) - set(index_names2)
    if missing_indexes:
        logger.warning(f"Table: {table} 中缺失的索引：{missing_indexes}")
    else:
        logger.info(f"Table: {table} 中没有缺失索引")

    # 获取索引交集
    common_indexes = set(index_names1) & set(index_names2)

    # 索引重新排序，第一序列Key_name，第二序列Seq_in_index
    indexes1 = sorted(indexes1, key=lambda x: (x["Key_name"], x["Seq_in_index"]))
    indexes2 = sorted(indexes2, key=lambda x: (x["Key_name"], x["Seq_in_index"]))

    # 去掉索引的基数
    for index in indexes1:
        if "Cardinality" in index:
            index.pop("Cardinality")
    for index in indexes2:
        if "Cardinality" in index:
            index.pop("Cardinality")

    # 将所有索引整合成一个元素，主要是处理复合多列索引
    index1_dict = dict()
    index2_dict = dict()
    for index in common_indexes:
        index1_dict[index] = [i for i in indexes1 if i["Key_name"] == index]
        index2_dict[index] = [i for i in indexes2 if i["Key_name"] == index]

    # 比较索引
    for index_name in common_indexes:
        index1 = index1_dict[index_name]
        index2 = index2_dict[index_name]
        if index1 != index2:
            logger.warning(f"Table: {table} 索引: {index_name} 的定义不一致")
            logger.warning(f"db1: {index1}")
            logger.warning(f"db2: {index2}")
        else:
            logger.info(f"Table: {table} 索引: {index_name} 的定义一致")


def compare_database(
    host1: str,
    port1: int,
    username1: str,
    password1: str,
    host2: str,
    port2: int,
    username2: str,
    password2: str,
    databases: list,
    charset1="utf8mb4",
    charset2="utf8mb4",
):
    """比较两个数据库的表结构，以db1为基准，比较db2中缺失的表，以及表结构的差异
    包括表名、字段名、字段类型、字段长度、字段默认值、字段是否为空、字段是否自增、字段是否主键、字段是否唯一、字段是否索引等一系统功能
    只要有任意一处不同，就输出不同的地方
    """
    connection1 = get_connection(host1, port1, username1, password1, charset=charset1)
    connection2 = get_connection(host2, port2, username2, password2, charset=charset2)

    try:
        with connection1.cursor() as cursor1, connection2.cursor() as cursor2:
            # 获取数据库中的Database
            db1_databases = get_databases(cursor1)
            db2_databases = get_databases(cursor2)
            logger.info(db1_databases)
            logger.info(db2_databases)

            for db in databases:
                # 判断数据库是否存在
                if db not in db1_databases:
                    logger.warning(f"Database: {db} 不存在于基准库【{host1}:{port1}】中，请检查")
                    continue

                if db not in db2_databases:
                    logger.warning(f"Database: {db} 不存在于比较库【{host2}:{port2}】中，跳过后续比较步骤")
                    continue

                cursor1.execute(f"USE `{db}`")
                cursor2.execute(f"USE `{db}`")

                # 比较层级一：database层面，比较database创建语句的差异
                sql_txt = f"SHOW CREATE DATABASE `{db}`"
                cursor1.execute(sql_txt)
                db1_define_sql = cursor1.fetchone()
                cursor2.execute(sql_txt)
                db2_define_sql = cursor2.fetchone()
                if db1_define_sql != db2_define_sql:
                    logger.warning(f"Database: {db} 的创建语句不一致")
                    logger.warning(f"db1: {db1_define_sql}")
                    logger.warning(f"db2: {db2_define_sql}")
                else:
                    logger.info(f"Database: {db} 的创建语句一致：{db1_define_sql}")

                # 比较层级二：table 数量层面，获取db2中缺少的表
                db1_tables = get_tables(cursor1, db)
                db2_tables = get_tables(cursor1, db)
                missing_tables = db1_tables - db2_tables
                if missing_tables:
                    logger.warning(
                        f"URL: {host2}:{port2} Database2: {db} 中缺失的表：{missing_tables}"
                    )
                else:
                    logger.info(f"URL: {host2}:{port2} Database2: {db} 中没有缺失表")

                # 比较层级三：table 定义层面
                common_tables = db1_tables & db2_tables
                for table in common_tables:
                    logger.info(f"比较表: {table}")

                    # 比较字段
                    cursor1.execute(f"SHOW COLUMNS FROM `{table}`")
                    cursor2.execute(f"SHOW COLUMNS FROM `{table}`")
                    columns1 = cursor1.fetchall()
                    columns2 = cursor2.fetchall()
                    compare_columns(table, columns1, columns2)

                    # 比较索引
                    cursor1.execute(f"SHOW INDEX FROM `{table}`")
                    cursor2.execute(f"SHOW INDEX FROM `{table}`")
                    indexes1 = cursor1.fetchall()
                    indexes2 = cursor2.fetchall()
                    compare_indexes(table, indexes1, indexes2)
    finally:
        # 关闭数据库连接
        connection1.close()
        connection2.close()


if __name__ == "__main__":
    pass



#!/usr/bin/env python
# coding:utf-8
"""功能简要说明
作者：dengqingyong
邮箱：yu12377@163.com
时间：2018/1/12 下午6:20
"""
import uuid
import time
import random
import string
import datetime
from typing import Tuple
from xpinyin import Pinyin

from .addressinfo import addr
from .bankinfo import bank_bin_list


def get_random_data(
    start_date: str = "", end_date: str = "", _format: str = "%Y-%m-%d"
) -> str:
    """
    随机生成日期
    :param start_date: 起始日期，格式为"%Y-%m-%d"
    :param end_date: 截止日期，格式为"%Y-%m-%d"
    :param _format: 输出格式，默认为"%Y%m%d"
    :return: 随机日期，格式为format
    """
    if not start_date:
        start_date = datetime.date(1949, 1, 1)
    else:
        start_date = datetime.datetime.strptime(start_date, _format)
    if not end_date:
        end_date = datetime.datetime.now().date()
    else:
        end_date = datetime.datetime.strptime(end_date, _format)
    diff_days = (end_date - start_date).days + 1
    random_days = random.randint(0, diff_days - 1)
    random_date = start_date + datetime.timedelta(days=random_days)
    return random_date.strftime(_format)


def get_random_string(length: int = 32, source: str = "dlu") -> str:
    """获取随机字符串
    :param length: int, 返回字符数量
    :param source: str, d代表数字，l代表26个小写字母，u代表26个大写字母，s代表所有
    """
    raw_str = ""
    # s 包含所有字符，故直接重新赋值
    if "s" in source:
        raw_str = string.printable[:-6]
    else:
        if "d" in source:
            raw_str += string.digits
        if "l" in source:
            raw_str += string.ascii_lowercase
        if "u" in source:
            raw_str += string.ascii_uppercase
    return "".join(random.choices(raw_str, k=length))


def get_uuid(length: int = 32) -> str:
    _uuid36 = str(uuid.uuid1())
    _uuid32 = _uuid36.replace("-", "")
    if length == 36:
        return _uuid36
    elif length == 32:
        return _uuid32
    else:
        return _uuid32[:length]


def get_pinyin(name: str = "", joint_mark="") -> Tuple[str, str]:
    """获取拼音
    :param name: str, 需要转换拼音的汉字
    :param joint_mark: str, 拼音连接符号
    :return 返回拼音与源汉字的元组
    """
    if not name:
        name = get_name()

    p = Pinyin()
    username = p.get_pinyin(name, joint_mark)
    return username, name


def get_name(gender=""):
    """获取中国人惯用姓名"""
    firstnames = """
        赵钱孙李，周吴郑王。
        冯陈褚卫，蒋沈韩杨。
        朱秦尤许，何吕施张。
        孔曹严华，金魏陶姜。
        戚谢邹喻，柏水窦章。
        云苏潘葛，奚范彭郎。
        鲁韦昌马，苗凤花方。
        俞任袁柳，酆鲍史唐。
        费廉岑薛，雷贺倪汤。
        滕殷罗毕，郝邬安常。
        乐于时傅，皮卞齐康。
        伍余元卜，顾孟平黄。
        和穆萧尹，姚邵湛汪。
        祁毛禹狄，米贝明臧。
        计伏成戴，谈宋茅庞。
        熊纪舒屈，项祝董梁。
        杜阮蓝闵，席季麻强。
        贾路娄危，江童颜郭。
        梅盛林刁，钟徐邱骆。
        高夏蔡田，樊胡凌霍。
        虞万支柯，昝管卢莫。
        经房裘缪，干解应宗。
        丁宣贲邓，郁单杭洪。"""

    firstnames = (
        firstnames.replace("，", "").replace("。", "").replace("\n", "").replace(" ", "")
    )
    firstname = random.choice(firstnames)

    secondname_boy = """
    澄邈、德泽、海超、海阳、海荣、海逸、海昌、瀚钰、瀚文、涵亮、涵煦、涵蓄、涵衍、浩皛、浩波、浩博、浩初、浩宕、浩歌、浩广、浩邈、浩气、
    浩思、浩言、鸿宝、鸿波、鸿博、鸿才、鸿畅、鸿畴、鸿达、鸿德、鸿飞、鸿风、鸿福、鸿光、鸿晖、鸿朗、鸿文、鸿轩、鸿煊、鸿骞、鸿远、鸿云、
    鸿哲、鸿祯、鸿志、鸿卓、嘉澍、光济、澎湃、彭泽、鹏池、鹏海、浦和、浦泽、瑞渊、越泽、博耘、德运、辰宇、辰皓、辰钊、辰铭、辰锟、辰阳、
    辰韦、辰良、辰沛、晨轩、晨涛、晨濡、晨潍、鸿振、吉星、铭晨、起运、运凡、运凯、运鹏、运浩、运诚、运良、运鸿、运锋、运盛、运升、运杰、
    运珧、运骏、运凯、运乾、维运、运晟、运莱、运华、耘豪、星爵、星腾、星睿、星泽、星鹏、星然、震轩、震博、康震、震博、振强、振博、振华、
    振锐、振凯、振海、振国、振平、昂然、昂雄、昂杰、昂熙、昌勋、昌盛、昌淼、昌茂、昌黎、昌燎、昌翰、晨朗、德明、德昌、德曜、范明、飞昂、
    高旻、晗日、昊然、昊天、昊苍、昊英、昊宇、昊嘉、昊明、昊伟、昊硕、昊磊、昊东、鸿晖、鸿朗、华晖、金鹏、晋鹏、敬曦、景明、景天、景浩、
    俊晖、君昊、昆琦、昆鹏、昆纬、昆宇、昆锐、昆卉、昆峰、昆颉、昆谊、昆皓、昆鹏、昆明、昆杰、昆雄、昆纶、鹏涛、鹏煊、曦晨、曦之、新曦、
    旭彬、旭尧、旭鹏、旭东、旭炎、炫明、宣朗、学智、轩昂、彦昌、曜坤、曜栋、曜文、曜曦、曜灿、曜瑞、智伟、智杰、智刚、智阳、昌勋、昌盛、
    昌茂、昌黎、昌燎、昌翰、晨朗、昂然、昂雄、昂杰、昂熙、范明、飞昂、高朗、高旻、德明、德昌、德曜、智伟、智杰、智刚、智阳、瀚彭、旭炎、
    宣朗、学智、昊然、昊天、昊苍、昊英、昊宇、昊嘉、昊明、昊伟、鸿朗、华晖、金鹏、晋鹏、敬曦、景明、景天、景浩、景行、景中、景逸、景彰、
    昆鹏、昆明、昆杰、昆雄、昆纶、鹏涛、鹏煊、景平、俊晖、君昊、昆琦、昆鹏、昆纬、昆宇、昆锐、昆卉、昆峰、昆颉、昆谊、轩昂、彦昌、曜坤、
    曜文、曜曦、曜灿、曜瑞、曦晨、曦之、新曦、鑫鹏、旭彬、旭尧、旭鹏、旭东、浩涆、浩瀚、浩慨、浩阔、鸿熙、鸿羲、鸿禧、鸿信、泽洋、泽雨、
    哲瀚、胤运、佑运、允晨、运恒、运发、云天、耘志、耘涛、振荣、振翱、中震、子辰、晗昱、瀚玥、瀚昂、瀚彭、景行、景中、景逸、景彰、绍晖、
    文景、曦哲、永昌、子昂、智宇、智晖、晗日、晗昱、瀚玥、瀚昂、昊硕、昊磊、昊东、鸿晖、绍晖、文昂、文景、曦哲、永昌、子昂、智宇、智晖、
    浩然、鸿运、辰龙、运珹、振宇、高朗、景平、鑫鹏、昌淼、炫明、昆皓、曜栋、文昂"""

    secondname_grid = """
    恨桃、依秋、依波、香巧、紫萱、涵易、忆之、幻巧、水风、安寒、白亦、惜玉、碧春、怜雪、听南、念蕾、紫夏、凌旋、芷梦、凌寒、梦竹、千凡、
    采波、元冬、思菱、平卉、笑柳、雪卉、南蓉、谷梦、巧兰、绿蝶、飞荷、平安、芷荷、怀瑶、慕易、若芹、紫安、曼冬、寻巧、寄波、尔槐、以旋、
    初夏、依丝、怜南、傲菡、谷蕊、笑槐、飞兰、笑卉、迎荷、元冬、痴安、妙绿、觅雪、寒安、沛凝、白容、乐蓉、映安、依云、映冬、凡雁、梦秋、
    梦凡、秋巧、若云、元容、怀蕾、灵寒、天薇、翠安、乐琴、宛南、怀蕊、白风、访波、亦凝、易绿、夜南、曼凡、亦巧、青易、冰真、白萱、友安、
    海之、小蕊、又琴、天风、若松、盼菡、秋荷、香彤、语梦、惜蕊、迎彤、沛白、雁山、易蓉、雪晴、诗珊、春冬、又绿、冰绿、半梅、笑容、沛凝、
    映秋、盼烟、晓凡、涵雁、问凝、冬萱、晓山、雁蓉、梦蕊、山菡、南莲、飞双、凝丝、思萱、怀梦、雨梅、冷霜、向松、迎丝、迎梅、雅彤、香薇、
    以山、碧萱、寒云、向南、书雁、怀薇、思菱、忆文、翠巧、怀山、若山、向秋、凡白、绮烟、从蕾、天曼、又亦、从安、绮彤、之玉、凡梅、依琴、
    沛槐、又槐、元绿、安珊、夏之、易槐、宛亦、白翠、丹云、问寒、易文、傲易、青旋、思真、雨珍、幻丝、代梅、盼曼、妙之、半双、若翠、初兰、
    惜萍、初之、宛丝、寄南、小萍、静珊、千风、天蓉、雅青、寄文、涵菱、香波、青亦、元菱、翠彤、春海、惜珊、向薇、冬灵、惜芹、凌青、谷芹、
    雁桃、映雁、书兰、盼香、向山、寄风、访烟、绮晴、映之、醉波、幻莲、谷冬、傲柔、寄容、以珊、紫雪、芷容、书琴、寻桃、涵阳、怀寒、易云、
    代秋、惜梦、尔烟、谷槐、怀莲、夜山、芷卉、向彤、新巧、语海、灵珊、凝丹、小蕾、迎夏、慕卉、飞珍、冰夏、亦竹、飞莲、海白、元蝶、春蕾、
    怀绿、尔容、小玉、幼南、凡梦、碧菡、初晴、宛秋、傲旋、新之、凡儿、夏真、静枫、痴柏、恨蕊、乐双、念薇、靖雁、寄松、丹蝶、元瑶、冰蝶、
    念波、迎松、海瑶、乐萱、凌兰、曼岚、若枫、傲薇、凡灵、乐蕊、秋灵、谷槐、觅云、寻春、恨山、从寒、忆香、觅波、静曼、青寒、笑天、涵蕾、
    元柏、代萱、紫真、千青、雪珍、寄琴、绿蕊、醉柳、诗翠、念瑶、孤风、曼彤、怀曼、香巧、采蓝、芷天、尔曼、巧蕊"""

    # 随机获取男名与女名
    grid_name = random.choice(
        secondname_grid.replace("\n", "").replace(" ", "").split("、")
    )
    boy_name = random.choice(
        secondname_boy.replace("\n", "").replace(" ", "").split("、")
    )

    # 将两个字的名字随机转换成双字或单字
    if random.randint(1, 3) == 1:
        grid_name = random.choice(grid_name)
        boy_name = random.choice(boy_name)

    # 男名or女名
    if gender.lower() in ("girl", "woman"):
        secondname = grid_name
    elif gender.lower() in ("boy", "man"):
        secondname = boy_name
    elif isinstance(gender, int):
        if gender % 2 == 0:
            secondname = grid_name
        else:
            secondname = boy_name
    else:
        secondname = random.choice([boy_name, grid_name])

    return "{}{}".format(firstname, secondname)


def check_idcards(idcards):
    """函数功能：
    校验模式：1. 校验中国大陆身份证号码的第18位是否正确；
    计算模式：2. 传入前17位，计算第18位校验码返回；
    返回为空时代码校验不通过；
    返回有值时代码校验通过，或是计算模式；
    """
    check_code = ""
    # 统一转换成字符串类型
    try:
        idcards = str(idcards)
    except TypeError as e:
        pass

    # 判断传入的身份证号码长度，如果为18位时顺便截取校验码
    if len(idcards) == 18:
        check_code = idcards[-1:]
        idcards = idcards[:-1]
    elif len(idcards) == 17:
        pass
    else:
        print("输入的身份证号码【{}】位数有误，必须是18位！".format(idcards))
        return check_code

    # 至此已经被转成17位
    # 十七位数字本体码权重
    weight = [7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2]
    # 对应校验码字符值
    validate = ["1", "0", "X", "9", "8", "7", "6", "5", "4", "3", "2"]

    _sum = 0
    _mode = 0
    for i in range(0, 17):
        try:
            _sum = _sum + int(idcards[i]) * weight[i]
        except TypeError as e:
            print("输入的身份证号码【{}】前17位必须是数字！".format(idcards))
    _mode = _sum % 11
    # 计算模式
    if check_code == "":
        return validate[_mode]
    # 校验模式——校验通过
    elif check_code == validate[_mode]:
        return check_code
    # 校验模式——校验不通过
    else:
        return ""


def get_idcards(sex=0):
    """获取中国大陆18位身份证号码"""
    # part1: 随机获取地区编码
    area_code = random.choice(addr)[0]

    # part2: 随机获取1949/10/01到当天的一个日期
    # 日期起始时间: 1949/10/01 00:00:00
    birthdate = get_random_data(_format="%Y%m%d")

    # part3: 随机获取两位顺序码
    sequence_code = random.randrange(11, 99)

    # part4：性别编码：奇怪代表男性，偶数代码女性
    if sex == 0:
        sex_code = random.randrange(1, 10)
    elif sex % 2 == 1:
        sex_code = random.randrange(1, 10, 2)
    else:
        sex_code = random.randrange(2, 10, 2)

    # part5: 计算校验码
    check_code = check_idcards(
        "{}{}{}{}".format(area_code, birthdate, sequence_code, sex_code)
    )

    return "{}{}{}{}{}".format(
        area_code, birthdate, sequence_code, sex_code, check_code
    )


def get_mobile_no():
    """生成随机手机号码"""
    phone_head = [
        130,
        131,
        132,
        133,
        134,
        135,
        136,
        137,
        138,
        139,
        150,
        151,
        152,
        155,
        158,
        170,
        171,
        172,
        173,
        174,
        175,
        176,
        177,
        178,
        179,
        181,
        186,
        187,
        188,
        189,
    ]
    phone_no = str(random.choice(phone_head)) + "".join(
        random.choice("0123456789") for i in range(8)
    )
    return phone_no


def get_phone_serial_no():
    """生成手机串号"""
    serial_no = "".join(random.choice(string.ascii_uppercase) for i in range(4))
    serial_no += "-" + "".join(random.choice(string.ascii_uppercase) for i in range(4))
    serial_no += "-" + "".join(random.choice(string.digits) for i in range(5))
    return serial_no


def get_bank_no(num=1, bank=None, bank_name=None, ftype=None, length=None, first=None):
    """生成银联卡卡号
    :param bank: 银行简称，大写字母，如工行ICBC，建行CCB，农行ABC等——非必填，默认随机
    :param ftype: 卡片类型，储蓄卡DC，信用卡CC——非必填，默认随机
    :param length: 卡号长度，信用卡基本上都是16位，储蓄卡通常16至19位，最长19位，但偶尔有低有16位的——非必填，默认随机
    :param num: 一次生成的卡号数量——非必填，默认1
    :param first: 为True时返回一个对象，非True时返回列表
    """

    if length and (int(length) < 16 or int(length) > 19):
        raise ValueError("银联卡号通常是16到19数字，请检查输入的length参数")

    # 获取参数获取bin码
    bin_list = get_bank_bin(
        num=num, bank=bank, bank_name=bank_name, ftype=ftype, length=length
    )

    for bin in bin_list:
        __get_bank_no(bin_obj=bin)
    if first:
        return bin_list[0]
    else:
        return bin_list


def __get_bank_no(bin_obj):
    if not bin_obj["bin"].isdigit():
        raise ValueError("银行卡BIN应该为6位数字")

    # 中间数字长度=长度 - 6位bin长度 - 末位校验码
    part2_length = int(bin_obj["length"]) - len(bin_obj["bin"]) - 1
    bank_part2 = ""
    for i in range(part2_length):
        bank_part2 += str(random.randint(0, 9))

    # 计算末尾校验码
    bank_no = bin_obj["bin"] + bank_part2
    fsum = 0
    _index = 0
    while True:
        _index -= 1
        try:
            _value = int(bank_no[_index])
        except IndexError as e:
            break
        # 判断位置
        if abs(_index) % 2 == 1:
            # 倒数：奇数位时*2
            _value *= 2
            if _value >= 10:
                # 十位数时：取个位数字与十位数字相加
                fsum += _value - 9
            else:
                # 个位数时：直接采用
                fsum += _value
        else:
            # 倒数：偶数位时直接相加
            fsum += _value
    # 拼接末尾校验码
    if fsum % 10 == 0:
        check_no = 0
    else:
        check_no = 10 - fsum % 10
    bank_no += str(check_no)
    bin_obj["no"] = bank_no
    return bin_obj


def get_bank_bin(
    num: int = 1,
    bank: str = None,
    bank_name: str = None,
    ftype: str = None,
    length: str = None,
):
    """获取银行卡bin码"""
    if length:
        length = str(length)
    bin_list = []
    # 根据条件过滤
    for _bin in bank_bin_list:
        if bank and _bin["bank"] != bank:
            continue
        if bank_name and not _bin["name"].startswith(bank_name):
            continue
        if ftype and _bin["type"] != ftype:
            continue
        if length and _bin["length"] != length:
            continue
        bin_list.append(_bin)
    if not bin_list:
        if ftype == "CC" and int(length) > 16:
            raise ValueError("找不到对应的bin码，信用卡通常是16位，请检查输入的length参数")
        raise ValueError("找不到对应的bin码，请检查输入是否正确！")
    if num <= 0:
        return bin_list
    else:
        return random.choices(bin_list, k=num)


def get_special_character(count: int):
    raw_str = (
        r"""①②③④⑤⑥⑦⑧⑨⑩⑪⑫⑬⑭⑮⑯⑰⑱⑲⑳⑴⑵⑶⑷⑸⑹⑺⑻⑼⑽⑾⑿⒀⒁⒂⒃⒄⒅⒆⒇⒈⒉⒊⒋⒌⒍⒎⒏⒐⒑⒒⒓⒔⒕⒖⒗⒘⒙⒚⒛㊀㊁㊂㊃㊄㊅㊆㊇㊈㊉"""
        r"""㈠㈡㈢㈣㈤㈥㈦㈧㈨㈩№½⅓⅔¼¾⅛⅜⅝⅞+-×÷﹢﹣±/=∥∠≌∽≦≧≒﹤﹥≈≡≠=≤≥<>≮≯∷∶∫∮∝∞∧∨∑∏∪∩∈∵∴⊥∥∠⌒⊙√∟⊿㏒㏑%‰ⅠⅡⅢⅣⅤⅥ"""
        r"""ⅦⅧⅨⅩⅪⅫⅰⅱⅲⅳⅴⅵⅶⅷⅸⅹⅺⅻΑΒΓΔΕΖΗΘΙΚΛΜΝΞΟΠΡΣΤΥΦΧΨΩαβγδεζνξοπρσηθικλμτυφχψω㋀㋁㋂㋃㋄㋅㋆㋇㋈㋉㋊㋋㏠"""
        r"""㏡㏢㏣㏤㏥㏦㏧㏨㏩㏪㏫㏬㏭㏮㏯㏰㏱㏲㏳㏴㏵㏶㏷㏸㏹㏺㏻㏼㏽㏾㍘㍙㍚㍛㍜㍝㍞㍟㍠㍡㍢㍣㍤㍥㍦㍧㍨㍩㍪㍫㍬㍭㍯㍰㊐㊊㊎㊍㊌㊋㊏"""
        r"""㊑㊒㊓㊔㊕㊖㊗㊘㊜㊝㊞㊟㊠㊡㊢㊩㊪㊫㊬㊭㊮㊯㊰㊙㊚㊛㊣㊤㊥㊦㊧㊨囍㈱㍿卐卍ォミ灬彡ツ♩♪♫♬¶♭♯♮∮‖§Ψ⒜⒝⒞⒟⒠⒡⒢⒣⒤⒥⒦⒧⒨⒩"""
        r"""⒪⒫⒬⒭⒮⒯⒰⒱⒲⒳⒴⒵ⓐⓑⓒⓓⓔⓕⓖⓗⓘⓙⓚⓛⓜⓝⓞⓟⓠⓡⓢⓣⓤⓥⓦⓧⓨⓩⒶⒷⒸⒹⒺⒻⒼⒽⒾⒿⓀⓁⓂⓃⓄⓅⓆⓇⓈⓉⓊⓋⓌⓍⓎⓏ零壹贰叁伍陆柒"""
        r"""捌玖佰仟万亿☀☼♨☁☂☽☾❄❅❆☃©®℃℉♂♀㎡℗Ω㏎￼㎎㎏㎜㎝㎞㎡㏄㏎㏑㏒㏕℡%‰°′″￠℅￥$€￡₴$₰¢₤₳₲₪₵₣₱฿¤₡₮₭₩ރ₢₥₫₦zł﷼₠₧₯"""
        r"""₨Kčर₹ƒ₸￠┏┳┓┌┬┐╔╦╗╓╥╖╒╤╕╭╮╱╲─│┱┲╃╄┣╋┫├┼┤╠╬╣╟╫╢╞╪╡╰╯╲╱━┇┅┋┗┻┛└┴┘╚╩╝╙╨╜╘╧╛═║︴﹏﹋﹌✱✲✳❃✾✽✼✻✺✹✸✷"""
        r"""✶✵✴❄❅❆❇❈❉❊❋✱❤♡♥❥♠♣♤ღ❣★☆✡✦✧✩✪✫✬✭✮✯✰☑✓✔√☓☒✘ㄨ✕✖✗❏❐❑❒▏▐░▒▓▔▕■□▢▣▤▥▦▧▨▩▪▫▬▭▮▯ˍ∎⊞⊟⊠⊡"""
        r"""⋄▱◆◇◈◧◨◩◪◫◙◘▀▁▂▃▄▅▆▇▉▊▋█▌▍▎▰⊙●○◕¤☪❂✪☻☼Θ⊖⊘⊕⊚⊛⊜⊝◉◌◍◐◑◒◓◔⊗◖◗◯◤◥◄►▶◀◣◢▲▼▸◂▴▾△▽▷◁⊿▻◅▵▿▹◃∆◬◭◮∇☢"""
        r"""乾☰兑☱离☲震☳巽☴坎☵艮☶坤☷☯。，、：∶；‘’“”〝〞ˆˇ﹕︰﹔﹖﹑·¨.¸;´？！～—｜‖＂〃｀@﹫¡¿﹏﹋︴々﹟#﹩$﹠&﹪%﹡﹢×﹦‐"""
        r"""￣¯―﹨˜﹍﹎＿-~（）〈〉‹›﹛﹜『』〖〗［］《》〔〕}」【】︵︷︿︹︽_︶︸﹀︺︾ˉ﹂﹄︼﹁﹃︻▲●□…→āáǎàōóǒòēéěèīíǐìūúǔ"""
        r"""ùǖǘǚǜüêɑńňǹɡㄅㄆㄇㄈㄉㄊㄋㄌㄍㄎㄏㄐㄑㄒㄓㄔㄕㄖㄗㄘㄙㄚㄛㄜㄝㄞㄟㄠㄡㄢㄣㄤㄥㄦㄧㄨㄩぁあぃいぅうぇえぉおかがきぎくぐ"""
        r"""けげこごさざしじすずせぜそぞただちぢつづてでとどなにぬねのはばぱひびぴふぶぷへべぺほぼぽまみむめもゃやゅゆょよらりるれろゎ"""
        r"""わゐゑをんゔゕゖ゚゛゜ゝゞゟ゠ァアィイゥウェエォオカガキギクグケゲコゴサザシジスズセゼソゾタダチヂッツヅテデトドナニヌネノ"""
        r"""ハパヒビピフブプヘベペホボポマミムメモャヤュユョヨラリルレロヮワヰヱヲンヴヵヶヷヸヹヺ・ーヽヾヿ㍿ㄱㄲㄳㄴㄵㄶㄷㄸㄹㄺㄻㄼ"""
        r"""ㄽㄾㄿㅀㅁㅂㅃㅄㅅㅆㅇㅈㅉㅊㅋㅌㅍㅎㅏㅐㅑㅒㅓㅔㅕㅖㅗㅘㅙㅚㅛㅜㅝㅞㅟㅠㅡㅢㅥㅦㅧㅨㅩㅪㅫㅬㅭㅮㅯㅰㅱㅲㅳㅴㅵㅶㅷㅸㅹㅺㅻ"""
        r"""ㅼㅽㅾㅿㆀㆁㆂㆃㆄㆅㆆㆇㆈㆉㆊАБВГДЕЁЖЗИЙКЛМНОПРСТУФХЦЧШЩЪЫЭЮЯабвгдеёжзийклмнопрстуфхцчшщъыьэюя←↑→↓↙↘↖↗↰↱"""
        r"""↲↳↴↵↶↺↻↷➝⇄⇅⇆⇇⇈⇉⇊⇋⇌⇍⇎⇏⇐⇑⇒⇓⇔⇕⇖⇗⇘⇙⇚⇛↯↹↔↕⇦⇧⇨⇩➫➬➩➪➭➮➯➱⏎➜➡➥➦➧➨➷➸➻➼➽➸➹➳➤➟➲➢➣➞⇪➚➘➙➛"""
        r"""➺⇞⇟⇠⇡⇢⇣⇤⇥↜↝♐➴➵➶↼↽↾↿⇀⇁⇂⇃↞↟↠↡↢↣↤↪↫↬↭↮↯↩⇜⇝↸↚↛↥↦↧↨✐✎✏✑✒✍✉✁✂✃✄✆✉☎☏☢☠☣✈☜☞☝✍☚☟✌♤♧♡♢♠♣♥♦"""
        r"""☀☁☂❄☃♨웃유❖☽☾☪✿♂♀✪✯☭➳卍卐√×■◆●○◐◑✙☺☻❀⚘♔♕♖♗♘♙♚♛♜♝♞♟♧♡♂♀♠♣♥❤⊙◎☺☻☼▧▨♨◐◑↔↕▪▒◊◦▣▤▥▦▩◘"""
        r"""◈◇♬♪♩♭♪の★☆→あぃ￡Ю〓§♤♥▶¤✲❈✿✲❈➹☀☂☁【】┱┲❣✚✪✣✤✥✦❉❥❦❧❃❂❁❀✄☪☣☢☠☭ღ▶▷◀◁☀☁☂☃☄★☆☇☈⊙☊☋☌☍ⓛⓞⓥⓔ╬"""
        r"""∴☀☆∷﹌の◎▶☺☻►◄▧▨♨◐◑↔↕↘▀▄█▌◦☼♪の☆→♧ぃ￡❤▒▬♦◊◦♠♣▣۰•❤•۰►◄▧▨♨▪▫☼♦⊙●○①⊕◎Θ⊙¤㊣★☆♀◆◇◣◢◥▲▼△▽⊿◤◥✐✌✍"""
        r"""✡☜☞⊙◎♥♡▪▫☼♦▐░▒▬♦◊◘◙◦☼♠♣▣▤▥▦▩◘◙◈✄☪☣☢☠☪♈ºº₪¤큐«»™♂✿♥☺☻｡◕‿◕｡｡◕‿◕｡◕‿-｡◉◞◟◉⊙‿⊙⊙▂⊙⊙０⊙⊙︿⊙⊙ω⊙⊙"""
        r"""﹏⊙⊙△⊙⊙▽⊙∩▂∩∩０∩∩︿∩∩ω∩∩﹏∩∩△∩∩▽∩●▂●●０●●︿●●ω●●﹏●●△●●▽●∪▂∪∪０∪∪︿∪∪ω∪∪﹏∪∪△∪∪▽∪≧▂≦≧０≦≧︿≦≧ω≦≧﹏≦≧△≦≧"""
        r"""▽≦＞▂＜＞０＜＞︿＜＞ω＜＞﹏＜＞△＜＞▽＜╯▂╰╯０╰╯︿╰╯ω╰╯﹏╰╯△╰╯▽╰＋▂＋＋０＋＋︿＋＋ω＋﹏＋＋△＋＋▽＋ˋ▂ˊˋ０ˊˋ︿ˊˋω"""
        r"""ˊˋ﹏ˊˋ△ˊˋ▽ˊˇ▂ˇˇ０ˇˇ︿ˇˇωˇˇ﹏ˇˇ△ˇˇ▽ˇ˙▂˙˙０˙˙︿˙˙ω˙˙﹏˙˙△˙˙▽˙≡(▔﹏▔)≡⊙﹏⊙∥∣°ˋ︿ˊ﹀-#╯︿╰﹀(=‵′=)<(‵^′)>(ˉ▽ˉ"""
        r"""；)(-__-)b＼＿／￣□￣｜｜------\(˙<>˙)/------<(O>(‵▽′)ψ（°ο°）~@?(^人)?(＊?↓˙＊)(O^~^O)[>\/<]↓。υ。"""
        r"""↓(；°○°)(>c<)艹丶灬丨彡丿丬巛o氵刂卩s宀卩刂阝肀忄冫丿氵彡丬丨丩丬丶丷丿乀乁乂乄乆乛亅亠亻冂冫冖凵刂辶釒钅阝飠牜饣卩卪"""
        r"""厸厶厽孓宀巛巜彳廴彡彐彳忄扌攵氵灬爫犭疒癶礻糹纟罒罓耂艹訁覀兦亼亽亖亗吂凸凹卝卍卐匸皕旡玊尐幵"""
    )
    return "".join(random.choices(raw_str, k=count))



import re
import socket
import logging
import requests
import mimetypes
from typing import Union, Tuple
from pathlib import Path
from contextlib import closing
from urllib.parse import urlparse

from . import file as file_utils
from . import encrypt as encrypt_utils


logger = logging.getLogger("DengUtils")


def get_host_ip(reference: str = "") -> str:
    """获取主机ip地址
    :param reference: str, 参考地址，本地可能存在多个IP，获取能够访问此地址的IP
    """
    # 支持http或https地址，从中提取域名/主机地址
    absolute_http_url_regexp = re.compile(r"^https?://", re.I)
    if absolute_http_url_regexp.match(reference):
        url_obj = urlparse(reference)
        reference = url_obj.hostname
        target = url_obj.port
        if not target:
            if url_obj.scheme == "https":
                target = 443
            else:
                target = 80
    else:
        target = 80

    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        if reference:
            s.connect((reference, target))
        else:
            s.connect(("114.114.114.114", 80))
        ip = s.getsockname()[0]
        logger.debug(f"获取本机IP地址成功：{ip}")
    except Exception as e:
        logger.debug(e)
        logger.warning(f"获取本机IP地址失败")
        ip = ""
    finally:
        s.close()
    return ip


def is_valid_ip(ip):
    """Returns true if the given string is a well-formed IP address.
    Supports IPv4 and IPv6.
    """
    # IP地址必须是字符串
    if not isinstance(ip, str):
        return False

    if not ip or "\x00" in ip:
        # getaddrinfo resolves empty strings to localhost, and truncates
        # on zero bytes.
        return False

    try:
        res = socket.getaddrinfo(
            ip, 0, socket.AF_UNSPEC, socket.SOCK_STREAM, 0, socket.AI_NUMERICHOST
        )
        return bool(res)
    except socket.gaierror as e:
        if e.args[0] == socket.EAI_NONAME:
            return False
        raise


def verify_download_url(url: str, timeout=3):
    """校验URL下载地址是否存在"""
    try:
        res = requests.head(url, timeout=timeout)
    except Exception as e:
        return False
    else:
        if res.status_code < 400:
            return True
        else:
            return False


def download_file(url: str, file_save_path: Path):
    """下载文件"""
    # 检查存储文件夹是否存在，不存在时创建
    if not file_save_path.parent.exists():
        file_save_path.parent.mkdir(parents=True)
    # 覆盖提醒
    if file_save_path.exists():
        logger.warning(f"目标文件已经存在，直接覆盖！")
    # 开始下载
    with closing(requests.get(url, stream=True)) as _res:
        with open(file_save_path, mode="wb") as _app:
            for chunk in _res.iter_content(chunk_size=10 * 1024 * 1024):
                if chunk:
                    _app.write(chunk)
    # 检查下载结果
    if file_save_path.exists():
        logger.info(f"下载成功，保存路径：{file_save_path}")
        return True
    else:
        raise FileNotFoundError(f"下载失败：{url}")


def download_and_check_md5(url: str, target: Union[str, Path], md5: str = ""):
    """下载文件"""
    target = Path(target) if not isinstance(target, Path) else target

    # 检查需要下载的文件是否已经存在，如果存在且md5相同则直接使用
    if target.exists():
        if md5 and encrypt_utils.md5sum(_file_path=target) == md5:
            logger.info(f"本地已经存在目标文件，且md5校验通过，跳过下载：{target}")
            return
        target.unlink(missing_ok=True)

    # 检查下载目标存储路径是否存在，不存在时创建，防止后续报错
    if not target.parent.exists():
        target.parent.mkdir(parents=True)

    logger.debug(f"开始下载文件：{url}=>{target}")
    res = requests.get(url, stream=True)
    with open(target, mode="wb") as file:
        for packet in res.iter_content(chunk_size=5 * 1024 * 1024):
            if packet:
                file.write(packet)
    if md5:
        assert encrypt_utils.md5sum(_file_path=target) == md5
        logger.info(f"下载完成且md5校验通过：{target}")
        return
    file_utils.check_path_is_exits(target, path_type="file")
    logger.info(f"下载完成，跳过md5：{target}")


def is_port_available(_ip: str, _port: int) -> bool:
    """
    判断指定端口是否可用
    :param _ip: 要检查的IP地址
    :param _port: 要检查的端口号
    :return: 如果端口可用返回True，否则返回False
    """
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
        try:
            # 尝试绑定到本地地址和指定端口
            s.bind((_ip, _port))
            # 如果绑定成功，则认为端口可用
            return True
        except:
            return False


def find_available_port(_ip: str, _start_port: int, _end_port: int) -> int:
    """
    查找可用端口
    :param _ip: 要绑定的IP地址
    :param _start_port: 起始端口号
    :param _end_port: 结束端口号
    :return: 返回可用端口号
    """
    for _port in range(_start_port, _end_port + 1):
        if is_port_available(_ip, _port):
            return _port
    return 0


def convert_network_traffic(
    bytes_val: Union[int, float],
    in_type: str = "byte",
    target_unit: str = "auto",
    decimal: int = 2,
) -> Tuple[float, str]:
    """将字节单位的流量转换为更大的单位，注意是byte不是bit
    :param bytes_val: 字节单位的流量
    :param in_type: 输入的流量单位，可选值有["byte", "bit"]
    :param target_unit: 目标单位，可选值有["auto", "B", "KB", "MB", "GB", "TB"]
    :param decimal: 保留小数位数
    :return: 转换后的流量值，带单位
    """
    if in_type == "bit":
        units = ["b", "Kb", "Mb", "Gb", "Tb"]
    else:
        units = ["B", "KB", "MB", "GB", "TB"]
    factors = [1, 1024, 1024**2, 1024**3, 1024**4]

    if target_unit == "auto":
        for i in range(len(factors) - 1, -1, -1):  # 从最大的单位开始检查
            if bytes_val >= factors[i]:
                target_unit = units[i]
                bytes_val /= factors[i]
                break
        else:
            target_unit = "B"  # 当bytes_val小于1KB时选择B单位

    else:
        if target_unit not in units:
            raise ValueError(
                f"不支持的目标单位: {target_unit}，请规范输入，支持的单位有：{units}"
            )

        for i, unit in enumerate(units):
            if unit == target_unit:
                bytes_val /= factors[i]
                break

    formatted_val = str(round(bytes_val, decimal))
    if decimal == 0 and formatted_val.endswith(".0"):
        formatted_val = formatted_val[:-2]

    return float(formatted_val), target_unit


def get_file_content_type(file_path: Union[str, Path]) -> str:
    """获取文件的MIME类型"""
    mime_type, _ = mimetypes.guess_type(file_path)
    return mime_type



#!/usr/bin/env python
# coding:utf-8
"""将个人封装的公共方法打包"""
from setuptools import setup, find_packages
from deng import __version__


PACKAGE_NAME = "deng"
PACKAGE_VERSION = __version__

setup(
    name=PACKAGE_NAME,
    version=PACKAGE_VERSION,
    description="Personal method encapsulation",
    url="https://github.com/Deng2016/deng",
    author="dengqingyong",
    author_email="yu12377@163.com",
    packages=find_packages(),
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    exclude_package_data={"": [".gitignore", "lab.py"]},
    install_requires=[
        "requests>=2.26.0",
        "redis>=4.5.0",
        "pycryptodome>=3.15.0",
        "xpinyin>=0.7.6",
        "pysmb>=1.2.10",
        "pyjwt>=2.8.0",
        "pytz>=2025.2",
        "pymysql>=1.0.0",
    ],
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: Implementation",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Topic :: Software Development :: Libraries",
    ],
)



"""samba操作工具模块
包含功能：
1.获取服务器连接句柄；
2.创建目录；
3.上传文件；
"""
# coding = utf-8
import os
import json
import time
import logging
from pathlib import Path
from typing import Union
from smb.SMBConnection import SMBConnection
from smb.smb_structs import OperationFailure

from .encrypt import to_decode
from .file import check_path_is_exits


__all__ = ["get_server_handler", "create_directory", "upload_file", "COM_SAMBA", "TEST_SAMBA"]

IP_ADDRESS = None
USERNAME = None
PASSWORD = None

logger = logging.getLogger("DengUtils")

try:
    COM_SAMBA: dict = json.loads(to_decode(os.environ["COM_SAMBA"]))
    TEST_SAMBA: dict = json.loads(to_decode(os.environ["TEST_SAMBA"]))
except Exception as e:
    logger.warning(f"环境变量COM_SAMBA或TEST_SAMBA不存在：{str(e)}，启用空值")
    COM_SAMBA = dict()
    TEST_SAMBA = dict()


def get_server_handler(
    username="",
    password="",
    my_name="",
    remote_name="",
    ip_address="",
):
    """获取服务器连接句柄"""
    if not username:
        username = COM_SAMBA.get("username")

    if not password:
        password = COM_SAMBA.get("password")

    if not my_name:
        my_name = COM_SAMBA.get("my_name")

    if not remote_name:
        remote_name = COM_SAMBA.get("remote_name")

    if not ip_address:
        ip_address = COM_SAMBA.get("ip_address")

    if not username or not password or not my_name or not remote_name or not ip_address:
        error_msg = f"samba服务器信息不完整：{locals()}"
        logger.error(error_msg)
        raise ValueError(error_msg)

    conn = SMBConnection(
        username=username, password=password, my_name=my_name, remote_name=remote_name,
    )
    conn.connect(ip=ip_address)
    global IP_ADDRESS, USERNAME, PASSWORD
    IP_ADDRESS = ip_address
    USERNAME = username
    PASSWORD = password
    return conn


def get_server2_handler(
    username="",
    password="",
    my_name="",
    remote_name="",
    ip_address="",
):
    """获取服务器连接句柄"""
    if not username:
        username = TEST_SAMBA.get("username")

    if not password:
        password = TEST_SAMBA.get("password")

    if not my_name:
        my_name = TEST_SAMBA.get("my_name")

    if not remote_name:
        remote_name = TEST_SAMBA.get("remote_name")

    if not ip_address:
        ip_address = TEST_SAMBA.get("ip_address")

    if not username or not password or not my_name or not remote_name or not ip_address:
        error_msg = f"samba服务器信息不完整：{locals()}"
        logger.error(error_msg)
        raise ValueError(error_msg)

    conn = SMBConnection(
        username=username, password=password, my_name=my_name, remote_name=remote_name,
    )
    conn.connect(ip=ip_address)
    global IP_ADDRESS, USERNAME, PASSWORD
    IP_ADDRESS = ip_address
    USERNAME = username
    PASSWORD = password
    return conn


def create_directory(
    conn: SMBConnection, service_name: str, path: str, is_exist_ok=True
) -> bool:
    """创建目录，支持递归创建"""
    last_result = None
    dir_list = list(Path(path).parents)
    # 丢弃已经存在的根目录
    dir_list.pop()
    # 将目录本身插入
    dir_list.insert(0, Path(path))
    while dir_list:
        _path = dir_list.pop()
        try:
            conn.createDirectory(service_name=service_name, path=str(_path))
        except Exception as e:
            last_result = False
            if not is_exist_ok:
                raise e
        else:
            last_result = True
    return last_result


def upload_file(conn: SMBConnection, service_name: str, path: str, file_path: Union[str, Path]):
    logger.info(f"推送文件：{file_path} => {path}")
    # 防止多个终端同时推送同一个文件到同一地址导致的冲突
    # 重试30次，都失败时抛错
    count = 30
    while True:
        with open(file_path, "rb") as _file_obj:
            try:
                conn.storeFile(service_name=service_name, path=path, file_obj=_file_obj)
            except OperationFailure:
                count -= 1
                if count > 0:
                    logger.warning(f"上传文件失败：{path}，一秒后重试")
                    time.sleep(1)
                else:
                    raise
            else:
                break


def download_file(conn: SMBConnection, service_name: str, path: str, file_path: Union[str, Path]):
    with open(file_path, mode="wb") as _file:
        file_size = conn.retrieveFile(service_name, path, _file)
    logger.info(f"文件下载完成：大小{file_size}, {service_name}{path} => {file_path}")


def delete_file(conn: SMBConnection, service_name: str, path_file_pattern: str):
    _file_abs_path = f"{service_name}/{path_file_pattern}"
    try:
        conn.deleteFiles(service_name, path_file_pattern)
    except Exception as e:
        logger.warning(f"删除文件失败，文件不存在：{_file_abs_path}")
    else:
        logger.info(f"删除文件成功：{_file_abs_path}")


def pull_file_by_samba(file_path: Path, target_dir: str):
    """从samba服务器上下载文件
    :param file_path: Path,下载文件保存绝对路径
    :param target_dir: str,samba服务器上存储备份文件目录的相对地址
    """
    # samba服务上共享节点目录
    service_name = "share"
    # 连接仓库samba服务器
    conn = get_server_handler()
    # 下载文件
    target_file = target_dir + "/" + file_path.name
    download_file(conn, service_name, target_file, file_path)
    logger.info(f"文件下载成功：{service_name}{target_dir}=>{file_path}")


def push_file_to_samba(file_path: Path, target_dir: str):
    """推送文件到samba服务器上
    :param file_path: Path,本地文件绝对路径
    :param target_dir: str,samba服务器上存储备份文件目录的相对地址
    """
    global IP_ADDRESS
    check_path_is_exits(file_path, path_type="file")
    file_name = file_path.name
    # samba服务上共享节点目录
    service_name = "share"
    # 连接仓库samba服务器
    conn = get_server_handler()
    # 创建目录
    create_directory(conn, service_name, target_dir)
    # 上传新包
    upload_file(conn, service_name, f"{target_dir}/{file_name}", file_path)
    # 仓库完整路径
    package_repo_abs_path = (
        f"/{IP_ADDRESS}/{service_name}{target_dir}/{file_name}"
    )
    logger.info(f"备份成功：{file_path.stem}=>{package_repo_abs_path}")


if __name__ == "__main__":
    pass
